import { HttpClient, HTTP_INTERCEPTORS } from '@angular/common/http';
import { Injectable, EventEmitter, NgModule } from '@angular/core';
import { MappedGetAll, MappedHttpGet, MappedHttpPostLocation } from 'totvs-log-base-foundation';
import { of } from 'rxjs';
import { map } from 'rxjs/operators';
import { Router } from '@angular/router';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe de configuração para gerenciamento de Unidade
 */
class UnidadeConfigBase {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UnidadeGetAllService {
    /**
     * @param {?} httpClient
     * @param {?} unidadeConfig
     */
    constructor(httpClient, unidadeConfig) {
        this.httpClient = httpClient;
        unidadeConfig.urlUnidadeQuery.subscribe((/**
         * @param {?} urlUnidadeQuery
         * @return {?}
         */
        urlUnidadeQuery => {
            this._urlGetAll = `${urlUnidadeQuery}/unidades`;
            this.mappedGetAll = new MappedGetAll(this.httpClient, this.urlGetAll, {
                MapTo: (/**
                 * @param {?} unidade
                 * @return {?}
                 */
                (unidade) => unidade)
            }).withTotvsNoErrorHeader();
        }));
    }
    /**
     * @return {?}
     */
    get urlGetAll() {
        return this._urlGetAll;
    }
    /**
     * @return {?}
     */
    get page() {
        return this.mappedGetAll.page;
    }
    /**
     * @return {?}
     */
    get pageSize() {
        return this.mappedGetAll.pageSize;
    }
    /**
     * @return {?}
     */
    get hasNext() {
        return this.mappedGetAll.hasNext;
    }
    /**
     * @return {?}
     */
    get items() {
        return this.mappedGetAll.items;
    }
    /**
     * @return {?}
     */
    reset() {
        return this.mappedGetAll.reset();
    }
    /**
     * @return {?}
     */
    more() {
        return this.mappedGetAll.more();
    }
    /**
     * @return {?}
     */
    getAll() {
        return this.reset();
    }
}
UnidadeGetAllService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UnidadeGetAllService.ctorParameters = () => [
    { type: HttpClient },
    { type: UnidadeConfigBase }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UnidadeGetByIdService {
    /**
     * @param {?} httpClient
     * @param {?} unidadeConfig
     */
    constructor(httpClient, unidadeConfig) {
        this.httpClient = httpClient;
        unidadeConfig.urlUnidadeQuery.subscribe((/**
         * @param {?} urlUnidadeQuery
         * @return {?}
         */
        urlUnidadeQuery => {
            this._url = `${urlUnidadeQuery}/unidades/{0}`;
            this.mappedById = new MappedHttpGet(this.url, {
                MapTo: (/**
                 * @param {?} unidade
                 * @return {?}
                 */
                (unidade) => unidade)
            }, this.httpClient).withTotvsNoErrorHeader();
        }));
    }
    /**
     * @return {?}
     */
    get url() {
        return this._url;
    }
    /**
     * @param {?} unidadeId
     * @return {?}
     */
    Get(unidadeId) {
        return this.mappedById.Get(unidadeId);
    }
}
UnidadeGetByIdService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UnidadeGetByIdService.ctorParameters = () => [
    { type: HttpClient },
    { type: UnidadeConfigBase }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UnidadeService {
    /**
     * @param {?} unidadeGetAll
     */
    constructor(unidadeGetAll) {
        this.unidadeGetAll = unidadeGetAll;
        this._eventoUnidadeCorrenteDefinida = new EventEmitter();
    }
    /**
     * Unidade corrente
     * @return {?}
     */
    get unidadeCorrente() {
        return this._unidadeCorrente;
    }
    /**
     * Evento que é emitido quando a unidade corrente é definida
     * @return {?}
     */
    get eventoUnidadeCorrenteDefinida() {
        return this._eventoUnidadeCorrenteDefinida;
    }
    /**
     * Método que define a unidade corrente
     * @param {?} unidade
     * @return {?}
     */
    definirUnidadeCorrente(unidade) {
        this._unidadeCorrente = unidade;
        this.eventoUnidadeCorrenteDefinida.emit(this._unidadeCorrente);
    }
    /**
     * Método que busca e devolve a unidade corrente se não estiver definida
     * @return {?}
     */
    buscarUnidadeCorrente() {
        if (this.unidadeCorrente) {
            return of(this.unidadeCorrente);
        }
        else {
            return this.unidadeGetAll.reset().pipe(map((/**
             * @param {?} unidades
             * @return {?}
             */
            unidades => (unidades && unidades.length > 0) ? unidades[0] : undefined)));
        }
    }
}
UnidadeService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UnidadeService.ctorParameters = () => [
    { type: UnidadeGetAllService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Constante usado nas requisições para que a unidadeId seja inserida no filtro
 * @type {?}
 */
const SL_UNIDADE_ID_HEADER_PARAM = 'X-SL-Unidade-Id';
/**
 * Classe que adiciona a requisição o filtro por unidadeId
 */
class UnidadeIdInterceptorService {
    /**
     * Construtor que injeta o serviço de gerenciamento de unidade
     * @param {?} unidadeService
     */
    constructor(unidadeService) {
        this.unidadeService = unidadeService;
    }
    /**
     * Implementação da interface de interceptação HTTP
     * que apenas adiciona a requisição o filtro por unidadeId
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        if (request.headers.has(SL_UNIDADE_ID_HEADER_PARAM) && this.unidadeService.unidadeCorrente) {
            /** @type {?} */
            let params = request.params;
            if (request.method === 'GET') {
                params = params.append('unidadeId', this.unidadeService.unidadeCorrente.id);
            }
            /** @type {?} */
            let body = request.body;
            if (request.method === 'POST' && request.body !== null) {
                body = Object.assign({ unidadeId: this.unidadeService.unidadeCorrente.id }, request.body);
            }
            return next.handle(request.clone({
                headers: request.headers.delete(SL_UNIDADE_ID_HEADER_PARAM),
                params,
                body
            }));
        }
        else {
            return next.handle(request);
        }
    }
}
UnidadeIdInterceptorService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UnidadeIdInterceptorService.ctorParameters = () => [
    { type: UnidadeService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UnidadeIncluirService {
    /**
     * @param {?} httpClient
     * @param {?} unidadeConfig
     */
    constructor(httpClient, unidadeConfig) {
        this.httpClient = httpClient;
        unidadeConfig.urlUnidade.subscribe((/**
         * @param {?} urlUnidade
         * @return {?}
         */
        urlUnidade => {
            this._url = `${urlUnidade}/unidades`;
            this.mapped = new MappedHttpPostLocation(this.url, {
                MapTo: (/**
                 * @param {?} unidade
                 * @return {?}
                 */
                (unidade) => unidade)
            }, this.httpClient);
        }));
    }
    /**
     * @return {?}
     */
    get url() {
        return this._url;
    }
    /**
     * @param {?} unidade
     * @return {?}
     */
    Post(unidade) {
        return this.mapped.Post(unidade);
    }
}
UnidadeIncluirService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UnidadeIncluirService.ctorParameters = () => [
    { type: HttpClient },
    { type: UnidadeConfigBase }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Implementação de guard para identificação da unidade
 */
class UnidadeGuard {
    /**
     * @param {?} router
     * @param {?} servicoUnidade
     * @param {?} unidadeConfig
     */
    constructor(router, servicoUnidade, unidadeConfig) {
        this.router = router;
        this.servicoUnidade = servicoUnidade;
        this.unidadeConfig = unidadeConfig;
    }
    /**
     * @param {?} route
     * @param {?} state
     * @return {?}
     */
    canActivate(route, state) {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        resolve => {
            this.servicoUnidade.buscarUnidadeCorrente().subscribe((/**
             * @param {?} unidade
             * @return {?}
             */
            unidade => {
                if (unidade) {
                    this.servicoUnidade.definirUnidadeCorrente(unidade);
                    resolve(true);
                }
                else {
                    resolve(this.navegarParaRotaUnidadeNaoEncontrada(this.unidadeConfig.rotaUnidadeNaoEncontrada, state.url));
                }
            }), (/**
             * @param {?} error
             * @return {?}
             */
            error => {
                console.error(error);
                resolve(this.navegarParaRotaErroBuscarUnidade(this.unidadeConfig.rotaErroBuscarUnidade, state.url));
            }));
        }));
    }
    /**
     * @private
     * @param {?} rota
     * @param {?} rotaAtual
     * @return {?}
     */
    navegarParaRotaUnidadeNaoEncontrada(rota, rotaAtual) {
        if (rota && rota.trim() !== '' && `/${rota}` !== rotaAtual) {
            return this.router.parseUrl(`/${rota}`);
        }
        else {
            return true;
        }
    }
    /**
     * @private
     * @param {?} rota
     * @param {?} rotaAtual
     * @return {?}
     */
    navegarParaRotaErroBuscarUnidade(rota, rotaAtual) {
        if (rota && rota.trim() !== '') {
            if (`/${rota}` === rotaAtual) {
                return true;
            }
            else {
                return this.router.parseUrl(`/${rota}`);
            }
        }
        else {
            return false;
        }
    }
}
UnidadeGuard.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UnidadeGuard.ctorParameters = () => [
    { type: Router },
    { type: UnidadeService },
    { type: UnidadeConfigBase }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SLUnidadeModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: SLUnidadeModule,
            providers: [
                UnidadeService,
                UnidadeGetAllService,
                UnidadeGetByIdService,
                UnidadeIncluirService,
                UnidadeGuard
            ],
        };
    }
}
SLUnidadeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
                providers: [
                    {
                        provide: HTTP_INTERCEPTORS,
                        useClass: UnidadeIdInterceptorService,
                        multi: true
                    }
                ]
            },] }
];

export { SLUnidadeModule, SL_UNIDADE_ID_HEADER_PARAM, UnidadeConfigBase, UnidadeGetAllService, UnidadeGetByIdService, UnidadeGuard, UnidadeIdInterceptorService, UnidadeIncluirService, UnidadeService };
//# sourceMappingURL=totvs-log-base-foundation-src-sl-unidade.js.map
