import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { IGetAllQueryFilter } from './../get-all/get-all-query-filter.interface';
import { IMapper } from './../mappers/mapper.interface';
import { IQueryFilter } from './../query-filter.interface';
import { MappedHttp } from './mapped-http';
import { IGetEntity } from './verbs/get-entity.interface';
/**
 * Realiza a operação de Get para o enpoint informado, mapeando a entidade após recebe-la, com opção de filtros na string
 *
 * @template In Entidade recebida
 * @template Out Tipo em que a Entidade recebida será convertida
 */
export declare class MappedHttpGetWithQueryFilter<In, Out> extends MappedHttp implements IGetEntity<Out>, IGetAllQueryFilter {
    private mapper;
    private httpClient;
    private apiQuery;
    /** Propriedade que indica se deve ou não definir o cabeçalho 'X-Portinari-No-Error' */
    private _xTotvsNoError;
    readonly xTotvsNoError: boolean;
    /** Propriedade que indica se deve ou não definir o cabeçalho 'X-SL-Unidade-Id' */
    private _xSLUnidadeId;
    readonly xSLUnidadeId: boolean;
    /**
     * Método responsável por definir os filtros por query que devem ser aplicados a consulta
     * @param queryFilter Classe que contém a definição dos filtros
     */
    setQueryFilter(queryFilter: IQueryFilter): void;
    /**
     * Cria uma instância de MappedHttpGetWithQueryFilter.
     * @param endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param mapper o conversor para o endpoint
     * @param httpClient o cliente http
     */
    constructor(endpoint: string, mapper: IMapper<In, Out>, httpClient: HttpClient, urlMapper?: IMapper<Array<any>, string>);
    /**
     * Implementação da chamada de leitura de um recurso
     *
     * @param params parâmetros da url
     * @returns Deve ser assinado para realizar a leitura do serviço
     */
    Get(...params: Array<any>): Observable<Out>;
    /**
     * Retorna a instância definindo o cabeçalho para não apresentar as
     * mensagens de erro conforme padrão do Guia de Implementação de API
     * (X-Portinari-No-Error)
     */
    withTotvsNoErrorHeader(): MappedHttpGetWithQueryFilter<In, Out>;
    /**
     * Retorna a instância definindo o cabeçalho para injetar a unidadeId
     */
    withUnidadeId(): MappedHttpGetWithQueryFilter<In, Out>;
    /**
     * Método responsável por retornar os parâmetros de Query da consulta
     */
    private getQueryString;
    /** Monta o cabeçalho da requisição */
    private getHeaders;
}
