import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { IMapper } from './../mappers/mapper.interface';
import { MappedHttp } from './mapped-http';
import { IGetEntity } from './verbs/get-entity.interface';
/**
 * Realiza a operação de Get para o enpoint informado, mapeando a entidade antes de enviar
 *
 * @template In Entidade recebida
 * @template Out Tipo em que a Entidade recebida será convertida
 */
export declare class MappedHttpGet<In, Out> extends MappedHttp implements IGetEntity<Out> {
    private mapper;
    private httpClient;
    /** Propriedade que indica se deve ou não definir o cabeçalho 'X-Portinari-No-Error' */
    private _xTotvsNoError;
    readonly xTotvsNoError: boolean;
    /** Propriedade que indica se deve ou não definir o cabeçalho 'X-SL-Unidade-Id' */
    private _xSLUnidadeId;
    readonly xSLUnidadeId: boolean;
    /**
     * Creates an instance of MappedHttpGet.
     * @param endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param mapper o conversor para o endpoint
     * @param httpClient o cliente http
     */
    constructor(endpoint: string, mapper: IMapper<In, Out>, httpClient: HttpClient, urlMapper?: IMapper<Array<any>, string>);
    /**
     * Implementação da chamada de leitura de um recurso
     *
     * @param params parâmetros da url
     * @returns Deve ser assinado para realizar a leitura do serviço
     */
    Get(...params: Array<any>): Observable<Out>;
    /**
     * Retorna a instância definindo o cabeçalho para não apresentar as
     * mensagens de erro conforme padrão do Guia de Implementação de API
     * (X-Portinari-No-Error)
     */
    withTotvsNoErrorHeader(): MappedHttpGet<In, Out>;
    /**
     * Retorna a instância definindo o cabeçalho para injetar a unidadeId
     */
    withUnidadeId(): MappedHttpGet<In, Out>;
    /** Monta o cabeçalho da requisição */
    private getHeaders;
}
