import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { IMapper } from './../mappers/mapper.interface';
import { MappedHttp } from './mapped-http';
import { IPostEntity } from './verbs/post-entity.interface';
/**
 * Realiza a operação de Post para o enpoint informado, mapeando a entidade antes de enviar
 * Consumindo o ID a partir do header location
 * @template In Entidade recebida
 * @template Out Tipo em que a Entidade recebida será convertida
 */
export declare class MappedHttpPostLocation<In, Out> extends MappedHttp implements IPostEntity<In, string> {
    private mapper;
    private httpClient;
    /** Propriedade que indica se deve ou não definir o cabeçalho 'X-Portinari-No-Error' */
    private _xTotvsNoError;
    readonly xTotvsNoError: boolean;
    /** Propriedade que indica se deve ou não definir o cabeçalho 'X-SL-Unidade-Id' */
    private _xSLUnidadeId;
    readonly xSLUnidadeId: boolean;
    /**
     * Creates an instance of MappedHttpPost.
     * @param endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param mapper o conversor para o endpoint
     * @param httpClient o cliente http
     */
    constructor(endpoint: string, mapper: IMapper<In, Out>, httpClient: HttpClient, urlMapper?: IMapper<Array<any>, string>);
    /**
     * Implementação da chamada de criação de um recurso
     *
     * @param data dados de entrada para o serviço
     * @param params parâmetros da url
     * @returns Deve ser assinado para realizar a criação do recurso e receber o id
     */
    Post(data?: In, ...params: any[]): Observable<string>;
    /**
     * Retorna a instância definindo o cabeçalho para não apresentar as
     * mensagens de erro conforme padrão do Guia de Implementação de API
     * (X-Portinari-No-Error)
     */
    withTotvsNoErrorHeader(): MappedHttpPostLocation<In, Out>;
    /**
     * Retorna a instância definindo o cabeçalho para injetar a unidadeId
     */
    withUnidadeId(): MappedHttpPostLocation<In, Out>;
    /** Monta o cabeçalho da requisição */
    private getHeaders;
}
