import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { IMapper } from './../mappers/mapper.interface';
import { MappedHttp } from './mapped-http';
import { IPutEntity } from './verbs/put-entity.interface';
/**
 * Realiza a operação de Put para o enpoint informado, mapeando a entidade antes de enviar
 *
 * @template In Entidade recebida
 * @template Out Tipo em que a Entidade recebida será convertida
 * @template Response Estrutura de resposta esperada pelo backend
 */
export declare class MappedHttpPut<In, Out, Response> extends MappedHttp implements IPutEntity<In, Response> {
    private mapper;
    private httpClient;
    /** Propriedade que indica se deve ou não definir o cabeçalho 'X-Portinari-No-Error' */
    private _xTotvsNoError;
    readonly xTotvsNoError: boolean;
    /**
     * Creates an instance of MappedHttpPut.
     * @param endpoint aceita o formato /api/entity/{0}/other/{1}
     * @param mapper o conversor para o endpoint
     * @param httpClient o cliente http
     */
    constructor(endpoint: string, mapper: IMapper<In, Out>, httpClient: HttpClient, urlMapper?: IMapper<Array<any>, string>);
    /**
     * Implementação da chamada de atualização de um recurso
     *
     * @param data dados de entrada para o serviço
     * @param params parâmetros da url
     * @returns Deve ser assinado para realizar a atualização do recurso
     */
    Put(data?: In, ...params: any[]): Observable<Response>;
    /**
     * Retorna a instância definindo o cabeçalho para não apresentar as
     * mensagens de erro conforme padrão do Guia de Implementação de API
     * (X-Portinari-No-Error)
     */
    withTotvsNoErrorHeader(): MappedHttpPut<In, Out, Response>;
    /** Monta o cabeçalho da requisição */
    private getHeaders;
}
