export declare class JasmineMatchers {
    /**
     * Configura o comparador recursivo de objetos para todos os testes.
     * Para arrays, compara tamanho e ordem dos elementos
     * Para objetos, compara a existência das propriedades e valores do lado do expect para o lado do equal (não faz vice-versa)
     * Para valores, o valor e o tipo são iguais
     * Ele substitui a implementação do comparador expect().toEqual()
     * utilizar após o primeiro describe do teste
     * @example
     * describe('test description',() => {
     * JasmineMatchers.configureDeepObjectCompareAllTests();
     *   it('deveria ser igual', () => {
     *     expect({a:{b:[{c:'ABC'}]}}).toEqual({a:{b:[{c:'ABC'}]}}) //passes
     *   })
     * })
     */
    static configureDeepObjectCompareAllTests(): void;
    /**
     * Configura o comparador recursivo de objetos para a chamada atual.
     * Ele substitui a implementação do comparador expect().toEqual()
     * utilizar na primeira linha do it
     */
    static configureDeepObjectCompareSingleTest(): void;
    /**
     * A implementação do comparador. Ao alterar, a sugestão é criar outra função para ter outro comportamento
     */
    static recursiveCompare(first: any, second: any): boolean;
}
